/*  Motti -- a strategy game
    Copyright (C) 1999 Free Software Foundation

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
*/
#include <config.h>

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#ifdef HAVE_STRING_H
# include <string.h>
#else
# include <strings.h>
#endif

#include "wrappers.h"

extern void
die (const char *msg, ...)
{
  va_list args;
  char *fullmsg;
  int len;
  va_start (args, msg);

  len = strlen (msg)+1;
  fullmsg = my_malloc (len * sizeof (char));

  sprintf (fullmsg, "motti: %s\n", msg);
  vfprintf (stderr, fullmsg, args);
  va_end (args);
  free (fullmsg);
  exit (1);
}

extern char *
alloc_err_str (err_no, file_name)
     enum motti_errors err_no;
     const char *file_name;
{
  char *str;
  const char *msg[] =
  {
    "can't find map file: %s",
    "invalid map file: %s",
    "can't open display: %s\ntry motti -n for text mode",
    "can't load font: %s",
    /* Fix me.  */
    "malloc error",
    "can't find color: %s",
    "can't allocate a color: %s",
    "invalid mouse action: %s"
  };
#if 0
  const int msg_len[] =
  {
    22, 19, 48, 18, 13, 19, 25, 23
  };
#endif
  str = (char *) my_malloc ((sizeof (msg[err_no]) + file_name == NULL
			     ? strlen (file_name) : 0) * sizeof
			    (char));
  sprintf (str, msg[err_no], file_name);
  return str;
}

extern void *
my_malloc (size)
     size_t size;
{
  void *ptr;
  ptr = malloc(size);
  if (ptr == NULL && size != 0)
    {
      /* Function die uses my_malloc, so we can't use it here.  */
      fprintf (stderr, "motti: malloc error\n");
      exit (1);
    }
  return ptr;
}

extern void *
my_calloc (nmemb, size)
     size_t nmemb, size;
{
  void *ptr;
  ptr = calloc (nmemb, size);
  if (ptr == NULL && size != 0)
    {
      /* Function die uses my_malloc, so we can't use it here.  */
      fprintf (stderr, "motti: calloc error\n");
      exit (1);
    }
  return ptr;
}
